API_URL_DATA = ".renoweb.dk/Legacy/JService.asmx/GetAffaldsplanMateriel_mitAffald"
API_URL_SEARCH = ".renoweb.dk/Legacy/JService.asmx/Adresse_SearchByString"

NON_SUPPORTED_ITEMS = [
    "Asbest",
    "Beholderservice",
    "Bestillerordning",
    "Henteordning for grene",
    "Trærødder og stammer",
    "Ingen tømningsdato fundet!",
    "Skal tilmeldes",
    "Bestil afhentning",
    "Farligt affald (skal bestilles)",
    "Storskrald (skal bestilles)",
    "Haveaffald (skal bestilles)",
]

SUPPORTED_ITEMS = {
    "restaffaldmadaffald": [
        "Restaffald-Madaffald",
        "Rest/mad",
        "Restaffald",
        "Rest - Mad",
        "Rest-/Madaffald",
        "Mad- og restaffald",
        "Mad/rest",
        "Rest-/madaffald",
        "Mad/Rest",
        "Mad og restaffald",
        "Mad-/ og restaffald",
        "Rest / Mad",
        "Rest Mad",
        "Energispand - Obligatorisk min. 1 spand",
        "Rest/madaffald",
        "Energibeholder (mad/rest)",
        "Rest- og Madaffald",
    ],
    "madaffald": [""],
    "batterier": ["Batterier"],
    "dagrenovation": ["Dagrenovation"],
    "elektronik": [""],
    "glas": ["Industri Genbrugeligt"],
    "metalglas": ["Glas og metal", "Metal-Glas", "Glas/Metal", "Metal og Glas"],
    "pappi": [
        "Plast MDK og papir",
        "PAPPI",
        "Plast/MD-karton/PP",
        "Plast/Papir",
        "Papir/Plast og kartoner",
        "Plast og Mad- & drikkekartoner/Papir",
        "Plast, Metal, MDK",
    ],
    "farligtaffald": [
        "Farligt affald",
        "Miljøkasser",
        "Farligt Affald, Rød boks",
        "Genbrugsbilen",
    ],
    "farligtaffaldmiljoboks": [
        "Farligt affald/Miljøboks",
        "Papir og glas/dåser",
        "Miljøboks",
    ],
    "flis": [
        "Flis",
        "Flishugning",
    ],
    "genbrug": [
        "Genbrug",
        "Genbrugsøer",
        "Genanvendeligt",
        "Genbrug 240 L",
        "Genbrugshal (1 stk.)",
    ],
    "jern": [
        "Jern",
    ],
    "papir": [
        "Papir",
        "Papir 660 L",
    ],
    "papirglasdaaser": [
        "Papir og glas/dåser",
    ],
    "papirmetal": [
        "Papir & Metal",
        "Papir/metal",
        "Metal/papir",
        "Papir og Metal",
    ],
    "pap": [
        "Pap",
    ],
    "plastmetal": [
        "Plast & Metal",
        "Plast, metal & MDK 660L",
        "Plast/ Metal",
        "Plast/MDK/Metal",
        "Plast-metal",
        "Plast/Metal",
    ],
    "restaffald": ["",],
    "storskrald": [
        "Storskrald",
        "Stort elektronik",
        "Storskrald og genbrug",
        "Pap og Storskrald",
    ],
    "storskraldogtekstilaffald": [
        "Storskrald og tekstilaffald",
    ],
    "haveaffald": [
        "Haveaffald, flishugning",
        "Haveaffald",
        "Trærødder og stammer",
        "Haveaffald - Frivilligt",
        "Haveaffald 140 L",
        "Frivillig Haveaffald tømmes fra mandag til onsdag",
    ],
    "papirglas": ["Papir og pap/Glas", "Glas-papir", "Papir, Pap/Glas"],
    "plastmadkarton": [
        "Plast/MDK",
        "Plast & mad- og drikkekartoner",
        "Plast/ Mad- og drikkekartoner",
        "Plast og MDK",
    ],
    "pappapirglasmetal": [
        "Pap, papir, tekstil & metal og glas",
        "4delt beholder",
        "Ressourcebeholder (pap/papir og glas/metal)",
    ],
    "plastmetalmadmdk": [
        "Plast/Metal/Mad- & drikkekartoner",
        "Plast, mad- og drikkekartoner og metal",
    ],
    "plastmdkglasmetal": [],
    "pappapir": ["Pap/Papir", "Papir/Pap", "Papir og Pap"],
    "tekstil": [
        "Tekstiler",
        "Standplads",
        "Tekstilaffald",
        "Miljøkasse/tekstiler",
        "Tekstil",
        "Tekstilposer (1 stk.)",
    ],
    "glasplast": [
        "Glas/Plast og MDK",
    ],
    "plastmetalpapir": [
        "Plast/Metal/Papir ",
        "Genbrugsspand - Obligatorisk min. 1 spand",
    ],
    "plast": ["Plast - Obligatorisk min. 1 spand", "Plast"],
}

NON_MATERIAL_LIST = [
    "Pap bundtet, havebolig (1 stk.)",
]

MATERIAL_LIST = {
    "restaffaldmadaffald": [""],
    "batterier": [
        "Batterier i pose på låg (1 stk.)",
    ],
    "dagrenovation": [
        "240 l. container 1 x ugentligt (12 stk.)",
    ],
    "elektronik": [
        "Småt elektronik i pose på låg (1 stk.)",
    ],
    "glas": [
        "240 L glas (1 stk.)",
        "Glas, 660 liter (2 stk.)",
    ],
    "metalglas": [
        "Glas/metal (1 stk.)",
        "240 l Glas/metal Egenløsning (1 stk.)",
        "Metal/Glas 240 l - 2-kammer (1 stk.)",
        "Glas/metal, afstand over 5 meter (1 stk.)",
        "240 L - Metal og Glas (1 stk.)",
        "240 l glas/metal - 4 tømninger (1 stk.)",
        "240 liter - Metal/Glas - tømmes hver 8. uge (1 stk.)",
    ],
    "pappi": [
        "Plast/papir (1 stk.)",
        "370 l Plast+MDK/Papir - Egenløsning (1 stk.)",
        "Papir/Plast og MDK 240 l - 2-kammer (1 stk.)",
        "Plast/papir, afstand over 5 meter (1 stk.)",
        "240 L - Plast og Papir (1 stk.)",
        "240L genbrug (1 stk.)",
        "240 l PMDK/PP 3 ugers tømning (1 stk.)",
        "240 liter - Plast, MDK, papir, pap – tømmes hver 2. uge (1 stk.)",
    ],
    "farligtaffald": [
        "",
    ],
    "farligtaffaldmiljoboks": [
        "Miljøkasse (1 stk.)",
        "Miljøboks",
        "Rød kasse henteordning (1 stk.)",
    ],
    "flis": [""],
    "jern": [
        "240 L metal (1 stk.)",
        "Metal, 660 liter (1 stk.)",
    ],
    "genbrug": [
        "Tekstiler",
        "Genbrug",
        "Genbrugsøer",
        "Genanvendeligt",
        "Genbrug 240 L",
        "Genbrugshal (1 stk.)",
    ],
    "papir": [
        "660 L papir (1 stk.)",
        "Papir, 660 liter (3 stk.)",
    ],
    "papirmetal": [
        "Metal og papir 240 L (ejer.kommune) (1 stk.)",
    ],
    "pap": [
        "Pap 240 L (villa) (1 stk.)",
        "240 l Pap Egenløsning (1 stk.)",
        "Pap 240 L (1 stk.)",
        "Pap 240 L (ejer:kommune) (h) (1 stk.)",
        "Pap - 240 l.  (1 stk.)",
        "Indsamling af pap (løst og beholder) (1 stk.)",
        "240 L - Pap - Skel (1 stk.)",
        "Beholder til pap (1 stk.)",
        "770 L pap (1 stk.)",
        "Pap, 660 liter (3 stk.)",
    ],
    "plastmetal": [
        "Plast, småt metal & MDK - 240 l. (1 stk.)",
    ],
    "plastmdkglasmetal": ["240 L todelt genbrugsspand hver 4. uge (1 stk.)",
    ],
    "plast": [
        "Plast og MDK 240 L (ejer.kommune) (1 stk.)",
        "Plast, 660 liter (2 stk.)",
    ],
    "storskrald": [
        "Storskrald - fortovsindsamling (1 stk.)",
        "Storskrald (1 stk.)",
        "Storskrald enfamiliehus (1 stk.)",
        "Storskrald",
        "Stort elektronik",
        "Storskrald og genbrug",
        "Pap og Storskrald",
        "Storskrald distrikt 8 (1 stk.)",
    ],
    "storskraldogtekstilaffald": [""],
    "haveaffald": [
        "Gen-Skel 0-2 meter (1 stk.)",
        "Haveaffald henteordning (1 stk.)",
    ],
    "papirglas": [
        "Papir/glas - 240 l. fortovsindsamling (1 stk.)",
    ],
    "plastmadkarton": [
        "Genbrugsbeholder PMDK/MG-240L/3uge (1 stk.)",
        "Genbrug henteordning (1 stk.)",
        "660 L plast, mad- og drikkekartoner (1 stk.)",
    ],
    "papirglasdaaser": [
        "240 L 2-delt Papir/glas-dåser en-familie (1 stk.)",
    ],
    "pappapirglasmetal": [
        "240L metal, glas, plast/papir gl. (1 stk.)",
        "240 l genbrug 2-kammer (2023) (1 stk.)",
        "4-kammer (240 l) (1 stk.)",
        "240 L 2-delt Papir/glas-dåser en-familie (1 stk.)",
    ],
    "plastmetalmadmdk": [""],
    "pappapir": [
        "Genbrugsbeholder PP-240L/6uge (1 stk.)",
    ],
    "tekstil": [
        "Tekstil pose (1 stk.)",
        "Tekstilpose tømning (1 stk.)",
        "Tekstilaffald (1 stk.)",
        "Tekstilpose 25 L (1 stk.)",
    ],
    "glasplast": [""],
    "plastmetalpapir": [""],
    "papirglasmetalplast": [
        "4-kammer (370 l) (1 stk.)",
        "Pap og papir/metal, glas og hård plast - 240 L (1 stk.)",
        "240 l genbrug låg i låg (1 stk.)"
    ],
    "madaffald": ["140 L madaffald (1 stk.)",],
    "restaffald": ["9505: 770 L restaffald uge (2 stk.)",],
}

ICON_LIST = {
    "batterier": "mdi:battery",
    "elektronik": "mdi:power-plug",
    "restaffaldmadaffald": "mdi:trash-can",
    "dagrenovation": "mdi:trash-can",
    "glas": "liquor",
    "metalglas": "mdi:glass-fragile",
    "pappi": "mdi:trash-can",
    "farligtaffald": "mdi:recycle",
    "farligtaffaldmiljoboks": "mdi:recycle",
    "flis": "mdi:tree",
    "genbrug": "mdi:recycle",
    "jern": "mdi:bucket",
    "papir": "mdi:file",
    "papirmetal": "mdi:delete-empty",
    "pap": "mdi:note",
    "plastmetal": "mdi:trash-can-outline",
    "storskrald": "mdi:table-furniture",
    "storskraldogtekstilaffald": "mdi:table-furniture",
    "haveaffald": "mdi:leaf",
    "papirglas": "mdi:greenhouse",
    "papirglasmetalplast": "mdi:trash-can",
    "plastmadkarton": "mdi:trash-can",
    "papirglasdaaser": "mdi:trash-can",
    "pappapirglasmetal": "mdi:trash-can",
    "plastmdkglasmetal": "mdi:trash-can",
    "plastmetalmadmdk": "mdi:trash-can",
    "pappapir": "mdi:file",
    "tekstil": "mdi:recycle",
    "glasplast": "mdi:trash-can",
    "plastmetalpapir": "mdi:trash-can",
    "plast": "mdi:trash-can",
    "madaffald": "mdi:trash-can",
    "restaffald": "mdi:trash-can",
}

NAME_LIST = {
    "batterier": "Batterier",
    "elektronik": "Elektronik",
    "dagrenovation": "Dagrenovation",
    "farligtaffald": "Farligt affald",
    "farligtaffaldmiljoboks": "Farligt affald & Miljøboks",
    "flis": "Flis",
    "genbrug": "Genbrug",
    "glas": "Glas",
    "glasplast": "Glas, Plast & Madkartoner",
    "haveaffald": "Haveaffald",
    "jern": "Metal",
    "madaffald": "Madaffald",
    "metalglas": "Metal & Glas",
    "pap": "Pap",
    "papir": "Papir",
    "papirglas": "Papir, Pap & Glas",
    "papirglasdaaser": "Papir, Glas & Dåser",
    "papirglasmetalplast": "Papir, Glas, Metal & Plast",
    "papirmetal": "Papir & Metal",
    "pappapir": "Pap & Papir",
    "pappapirglasmetal": "Pap, Papir, Glas & Metal",
    "pappi": "Papir & Plast",
    "plast": "Plast",
    "plastmadkarton": "Plast & Madkarton",
    "plastmdkglasmetal": "Plast, Madkarton, Glas & Metal",
    "plastmetal": "Plast & Metal",
    "plastmetalmadmdk": "Plast, Metal, Mad & Drikkekartoner",
    "plastmetalpapir": "Plast, Metal & Papir",
    "restaffald": "Restaffald",
    "restaffaldmadaffald": "Rest & Madaffald",
    "storskrald": "Storskrald",
    "storskraldogtekstilaffald": "Storskrald & Tekstilaffald",
    "tekstil": "Tekstilaffald",
}

NAME_ARRAY = list(NAME_LIST.keys())

MUNICIPALITIES_LIST = {
    "Aabenraa": "aabenraa",
    "Aalborg": "aalborg",
    "Albertslund": "albertslund",
    "Allerød": "allerod",
    "Billund": "billund",
    "Brøndby": "brondby",
    "Brønderslev": "bronderslev",
    "Dragør": "dragoer",
    "Egedal": "egedal",
    "Esbjerg": "esbjerg",
    "Faxe": "faxe",
    "Fredensborg": "fredensborg",
    "Frederiksberg": "frederiksberg",
    "Frederikssund": "frederikssund",
    "Gentofte": "gentofte",
    "Gladsaxe": "gladsaxe",
    "Glostrup": "glostrup",
    "Greve": "greve",
    "Gribskov": "gribskov",
    "Halsnæs": "halsnaes",
    "Hedensted": "hedensted",
    "Helsingør": "helsingor",
    "Herlev": "herlev",
    "Hillerød": "hillerod",
    "Hjørring": "hjoerring",
    "Horsens": "horsens",
    "Hvidovre": "hvidovre",
    "Høje-Taastrup": "htk",
    "Hørsholm": "hoersholm",
    "Jammerbugt": "jammerbugt",
    "Kerteminde": "kerteminde",
    "Køge": "koege",
    "Lejre": "lejre",
    "Lyngby-Taarbæk": "ltf",
    "Mariagerfjord": "mariagerfjord",
    "Næstved": "naestved",
    "Odsherred": "odsherred",
    "Randers": "randers",
    "Ringkøbing-Skjern": "rksk",
    "Ringsted": "ringsted",
    "Roskilde": "roskilde",
    "Rudersdal": "rudersdal",
    "Rødovre": "rk",
    "Samsø": "samsoe",
    "Slagelse": "slagelse",
    "Solrød": "solrod",
    "Sorø": "soroe",
    "Stevns": "stevns",
    "Svendborg": "svendborg",
    "Sønderborg": "sonderborg",
    "Tårnby": "taarnby",
    "Varde": "varde",
    "Vejen": "vejen",
    "Vordingborg": "vordingborg",
}

MUNICIPALITIES_ARRAY = list(MUNICIPALITIES_LIST.keys())

WEEKDAYS = ["Mandag", "Tirsdag", "Onsdag", "Torsdag", "Fredag", "Lørdag", "Søndag"]
WEEKDAYS_SHORT = ["Man", "Tir", "Ons", "Tor", "Fre", "Lør", "Søn"]



# ruff: noqa: F401
"""A Python Wrapper to communicate with AffaldDK API."""

from __future__ import annotations

from pyaffalddk.api import (
    GarbageCollection,
    AffaldDKGarbageTypeNotFound,
    AffaldDKNotSupportedError,
    AffaldDKNotValidAddressError,
    AffaldDKNoConnection,
)
from pyaffalddk.data import PickupEvents, PickupType, AffaldDKAddressInfo

from pyaffalddk.const import ICON_LIST, MUNICIPALITIES_ARRAY, NAME_ARRAY, NAME_LIST, WEEKDAYS, WEEKDAYS_SHORT

__title__ = "pyaffalddk"
__version__ = "2.0.28"
__author__ = "briis"
__license__ = "MIT"



"""This module contains the code to get garbage data from AffaldDK."""
from __future__ import annotations

import abc
import datetime as dt
from datetime import timezone
import json
import logging

from typing import Any

import aiohttp

from .const import (
    API_URL_DATA,
    API_URL_SEARCH,
    ICON_LIST,
    MATERIAL_LIST,
    MUNICIPALITIES_LIST,
    NAME_LIST,
    NON_MATERIAL_LIST,
    NON_SUPPORTED_ITEMS,
    SUPPORTED_ITEMS,
    WEEKDAYS,
)
from .data import PickupEvents, PickupType, AffaldDKAddressInfo

_LOGGER = logging.getLogger(__name__)


class AffaldDKNotSupportedError(Exception):
    """Raised when the municipality is not supported."""


class AffaldDKNotValidAddressError(Exception):
    """Raised when the address is not found."""


class AffaldDKNoConnection(Exception):
    """Raised when no data is received."""


class AffaldDKGarbageTypeNotFound(Exception):
    """Raised when new garbage type is detected."""


class AffaldDKAPIBase:
    """Base class for the API."""

    @abc.abstractmethod
    async def async_api_request(self, url: str) -> dict[str, Any]:
        """Override this."""
        raise NotImplementedError(
            "users must define async_api_request to use this base class"
        )


class AffaldDKAPI(AffaldDKAPIBase):
    """Class to get data from AffaldDK."""

    def __init__(self) -> None:
        """Initialize the class."""
        self.session = None

    async def async_api_request(self, url: str, body: str) -> dict[str, Any]:
        """Make an API request."""

        is_new_session = False
        if self.session is None:
            self.session = aiohttp.ClientSession()
            is_new_session = True

        headers = {"Content-Type": "application/json"}
        # self.session.headers.update(headers)

        async with self.session.post(
            url, headers=headers, data=json.dumps(body)
        ) as response:
            if response.status != 200:
                if is_new_session:
                    await self.session.close()

                if response.status == 400:
                    raise AffaldDKNotSupportedError("Municipality not supported")

                if response.status == 404:
                    raise AffaldDKNotSupportedError("Municipality not supported")

                if response.status == 503:
                    raise AffaldDKNoConnection("System API is currently not available")

                raise AffaldDKNoConnection(f"Error {response.status} from {url}")

            data = await response.text()
            if is_new_session:
                await self.session.close()

            json_data = json.loads(data)
            return json_data


class GarbageCollection:
    """Class to get garbage collection data."""

    def __init__(
        self,
        municipality: str,
        timezone: str = "Europe/Copenhagen",
        session: aiohttp.ClientSession = None,
        api: AffaldDKAPIBase = AffaldDKAPI(),
    ) -> None:
        """Initialize the class."""
        self._municipality = municipality
        self._timezone = timezone
        self._street = None
        self._house_number = None
        self._api = api
        self._data = None
        self._municipality_url = None
        self._address_id = None
        if session:
            self._api.session = session
        for key, value in MUNICIPALITIES_LIST.items():
            if key.lower() == self._municipality.lower():
                self._municipality_url = value
                break

    async def async_init(self) -> None:
        """Initialize the connection."""
        if self._municipality is not None:
            url = f"https://{self._municipality_url}{API_URL_SEARCH}"
            body = {
                "searchterm": f"{self._street} {self._house_number}",
                "addresswithmateriel": 1,
            }
            await self._api.async_api_request(url, body)

    async def get_address_id(
        self, zipcode: str, street: str, house_number: str
    ) -> AffaldDKAddressInfo:
        """Get the address id."""

        if self._municipality_url is not None:
            url = f"https://{self._municipality_url}{API_URL_SEARCH}"
            body = {"searchterm": f"{street} {house_number}", "addresswithmateriel": 7}
            # _LOGGER.debug("Municipality URL: %s %s", url, body)
            data: dict[str, Any] = await self._api.async_api_request(url, body)
            result = json.loads(data["d"])
            # _LOGGER.debug("Address Data: %s", result)
            if "list" not in result:
                raise AffaldDKNoConnection(
                    f"AffaldDK API: {result['status']['status']} - {result['status']['msg']}"
                )

            _result_count = len(result["list"])
            _item: int = 0
            _row_index: int = 0
            if _result_count > 1:
                for row in result["list"]:
                    if zipcode in row["label"]:
                        _item = _row_index
                        break
                    _row_index += 1
            self._address_id = result["list"][_item]["value"]

            if self._address_id == "0000":
                raise AffaldDKNotValidAddressError("Address not found")

            address_data = AffaldDKAddressInfo(
                self._address_id,
                self._municipality.capitalize(),
                street.capitalize(),
                house_number,
            )
            return address_data
        else:
            raise AffaldDKNotSupportedError("Cannot find Municipality")

    async def get_pickup_data(self, address_id: str) -> PickupEvents:
        """Get the garbage collection data."""

        if self._municipality_url is not None:
            self._address_id = address_id
            url = f"https://{self._municipality_url}{API_URL_DATA}"
            # _LOGGER.debug("URL: %s", url)
            body = {"adrid": f"{address_id}", "common": "false"}
            # _LOGGER.debug("Body: %s", body)
            data = await self._api.async_api_request(url, body)
            result = json.loads(data["d"])
            garbage_data = result["list"]
            # _LOGGER.debug("Garbage Data: %s", garbage_data)

            pickup_events: PickupEvents = {}
            _next_pickup = dt.datetime(2030, 12, 31, 23, 59, 0)
            _next_pickup = _next_pickup.date()
            _next_pickup_event: PickupType = None
            _next_name = []
            _next_description = []
            _last_update = dt.datetime.now(timezone.utc)
            _utc_date = _last_update.replace(tzinfo=timezone.utc)
            _utc_timestamp = _utc_date.timestamp()

            for row in garbage_data:
                if row["ordningnavn"] in NON_SUPPORTED_ITEMS:
                    continue

                _pickup_date = None
                if row["toemningsdato"] not in NON_SUPPORTED_ITEMS:
                    _pickup_date = to_date(row["toemningsdato"])
                elif str(row["toemningsdage"]).capitalize() in WEEKDAYS:
                    _pickup_date = get_next_weekday(row["toemningsdage"])
                else:
                    continue

                if any(
                        group in row["ordningnavn"].lower()
                        for group in [
                            "genbrug", "storskrald", "papir og glas/dåser","miljøkasse/tekstiler"
                        ]) and self._municipality.lower() == "gladsaxe":
                    key = get_garbage_type_from_material(row["materielnavn"], self._municipality, self._address_id)
                elif  any(
                        group in row["ordningnavn"].lower()
                        for group in [
                            "genbrug", "papir og glas/dåser","miljøkasse/tekstiler", "standpladser"
                        ]):
                    key = get_garbage_type_from_material(row["materielnavn"], self._municipality, self._address_id)
                else:
                    key = get_garbage_type(row["ordningnavn"])

                if key == row["ordningnavn"] and key != "Bestillerordning":
                    _LOGGER.warning(
                        "Garbage type [%s] is not defined in the system. Please notify the developer. Municipality: %s, Address ID: %s",
                        key, self._municipality, self._address_id
                    )
                    continue

                _pickup_event = {
                    key: PickupType(
                        date=_pickup_date,
                        group=row["ordningnavn"],
                        friendly_name=NAME_LIST.get(key),
                        icon=ICON_LIST.get(key),
                        entity_picture=f"{key}.svg",
                        description=row["materielnavn"],
                        last_updated=_utc_date,
                        utc_timestamp=_utc_timestamp,
                    )
                }
                pickup_events.update(_pickup_event)

                if _pickup_date is not None:
                    if _pickup_date < dt.date.today():
                        continue
                    if _pickup_date < _next_pickup:
                        _next_pickup = _pickup_date
                        _next_name = []
                        _next_description = []
                    if _pickup_date == _next_pickup:
                        _next_name.append(NAME_LIST.get(key))
                        _next_description.append(row["materielnavn"])

            _next_pickup_event = {
                "next_pickup": PickupType(
                    date=_next_pickup,
                    group="genbrug",
                    friendly_name=list_to_string(_next_name),
                    icon=ICON_LIST.get("genbrug"),
                    entity_picture="genbrug.svg",
                    description=list_to_string(_next_description),
                    last_updated=_utc_date,
                    utc_timestamp=_utc_timestamp,
                )
            }

            pickup_events.update(_next_pickup_event)
            return pickup_events


def to_date(datetext: str) -> dt.date:
    """Convert a date string to a datetime object."""
    if datetext == "Ingen tømningsdato fundet!":
        return None

    index = datetext.rfind(" ")
    if index == -1:
        return None
    _date = dt.datetime.strptime(f"{datetext[index+1:]}", "%d-%m-%Y")
    return _date.date()


def get_garbage_type(item: str) -> str:
    """Get the garbage type."""
    # _LOGGER.debug("Affalds type: %s", item)
    for key, value in SUPPORTED_ITEMS.items():
        if item.lower() in str(value).lower():
            for entry in value:
                if item.lower() == entry.lower():
                    return key
    return item


def get_garbage_type_from_material(item: str, municipality: str, address_id: str) -> str:
    """Get the garbage type from the materialnavn."""
    # _LOGGER.debug("Material: %s", item)
    for key, value in MATERIAL_LIST.items():
        if item in NON_MATERIAL_LIST:
            continue
        if item.lower() in str(value).lower():
            for entry in value:
                if item.lower() == entry.lower():
                    return key

    _LOGGER.warning(
        "Material type [%s] is not defined in the system for Genbrug. Please notify the developer. Municipality: %s, Address ID: %s",
        item, municipality, address_id
    )
    return "genbrug"


def get_next_weekday(weekday: str) -> dt.date:
    weekdays = WEEKDAYS
    current_weekday = dt.datetime.now().weekday()
    target_weekday = weekdays.index(weekday.capitalize())
    days_ahead = (target_weekday - current_weekday) % 7
    next_date: dt.date = dt.datetime.now() + dt.timedelta(days=days_ahead)
    return next_date


def list_to_string(list: list[str]) -> str:
    """Convert a list to a string."""
    return " | ".join(list)



import datetime as dt
from dataclasses import dataclass
import logging

_LOGGER = logging.getLogger(__name__)


@dataclass
class AffaldDKAddressInfo:
    """Represent AffaldDK address info."""

    address_id: str
    kommunenavn: str
    vejnavn: str
    husnr: str


@dataclass(frozen=True)
class PickupType:
    """Define a waste pickup type."""

    date: dt.date | None = None
    group: str | None = None
    friendly_name: str | None = None
    icon: str | None = None
    entity_picture: str | None = None
    description: str | None = None
    last_updated: dt.datetime | None = None
    utc_timestamp: int | None = None


@dataclass(frozen=True)
class PickupEvents:
    """Represent AffaldDK collection data."""

    batterier: list[PickupType] | None = None
    dagrenovation: list[PickupType] | None = None
    elektronik: list[PickupType] | None = None
    farligtaffald: list[PickupType] | None = None
    farligtaffaldmiljoboks: list[PickupType] | None = None
    flis: list[PickupType] | None = None
    genbrug: list[PickupType] | None = None
    glas: list[PickupType] | None = None
    glasplast: list[PickupType] | None = None
    haveaffald: list[PickupType] | None = None
    jern: list[PickupType] | None = None
    metalglas: list[PickupType] | None = None
    pap: list[PickupType] | None = None
    papir: list[PickupType] | None = None
    papirglas: list[PickupType] | None = None
    papirglasdaaser: list[PickupType] | None = None
    papirglasmetalplast: list[PickupType] | None = None
    papirmetal: list[PickupType] | None = None
    pappair: list[PickupType] | None = None
    pappapirglasmetal: list[PickupType] | None = None
    pappi: list[PickupType] | None = None
    plast: list[PickupType] | None = None
    plastmadkarton: list[PickupType] | None = None
    plastmetal: list[PickupType] | None = None
    plastmetalmadmdk: list[PickupType] | None = None
    plastmetalpapir: list[PickupType] | None = None
    restaffaldmadaffald: list[PickupType] | None = None
    storskrald: list[PickupType] | None = None
    storskraldogtekstilaffald: list[PickupType] | None = None
    tekstil: list[PickupType] | None = None
    next_pickup: list[PickupType] | None = None



"""This module contains the embedded images for Entity Pictures."""
from __future__ import annotations

PICTURE_ITEMS = {
    "batterier": "",
    "elektronik": "",
    "dagrenovation": "",
    "farligtaffald": "",
    "farligtaffaldmiljoboks": "",
    "flis": "",
    "genbrug": "",
    "glas": "",
    "glasplast": "",
    "haveaffald": "",
    "jern": "",
    "metalglas": "",
    "pap": "",
    "papir": "",
    "papirglas": "",
    "papirglasdaaser": "",
    "papirglasmetalplast": "",
    "papirmetal": "",
    "pappapir": "",
    "pappapirglasmetal": "",
    "pappi": "",
    "plast": "",
    "plastmadkarton": "",
    "plastmetal": "",
    "plastmetalmadmdk": "",
    "plastmetalpapir": "",
    "restaffaldmadaffald": "",
    "storskrald": "",
    "storskraldogtekstilaffald": "",
    "tekstil": "",
}
